"""
Module providing the ability to do the following:
    1. Read target data from ckl file.
    2. Write target data to the ckl file
    3. Write vkey data to the ckl file
    4. Load vkey data that can then be used to verify you are writing to an 
       actual vkey in that stig
    5. Load target data that can then be used to verify you are writing to 
       an actual target in that stig
"""

import json
import xmltodict
from stig_edit import handler

def read_target_data(file_name):

    '''Read CKL Target Data'''

    with open(file_name, 'r', encoding='utf-8') as ckl:
        ckl_dict = xmltodict.parse(ckl.read())

    target_data = json.dumps(ckl_dict['CHECKLIST']['ASSET'], indent=2)

    return target_data


def write_target_data(file_name, key, value):

    '''Write to CKL Target Data'''

    with open(file_name, 'r', encoding='utf-8') as ckl:
        ckl_dict = xmltodict.parse(ckl.read())

        target_values = ckl_dict['CHECKLIST']['ASSET'].keys()

    #Error out if the file is not found, is a directory, or incorrect permissions on the file
    handler.check_file_and_keys(file_name=file_name, key=key, available_inputs=target_values)

    #Set the new value
    with open(file_name, 'r+', encoding='utf-8') as ckl:
        ckl_dict = xmltodict.parse(ckl.read())
        ckl_dict['CHECKLIST']['ASSET'][key] = value
        print(f'Setting {key} to {value} in dictionary.')

    #Add to XML file
    with open(file_name, 'r+', encoding='utf-8') as ckl:

        xml_out = xmltodict.unparse(ckl_dict, pretty=True)
        ckl_file_data = [   '<?xml version="1.0" encoding="UTF-8"?>\n',
                            '<!--DISA STIG Viewer :: 2.17-->\n', 
                            xml_out
                        ]
        ckl.writelines(ckl_file_data)
        print(f"Adding {key}'s value from dictionary to ckl file.")

    #Create a list from all the lines to remove duplicate xml declarations
    with open(file_name, 'r', encoding='utf-8') as ckl:
        lines = ckl.readlines()

    #Rewrite the file without duplicate xml declaration
    with open(file_name, 'w', encoding='utf-8') as ckl:
        for number,line in enumerate(lines):
            if number not in [2]:
                ckl.writelines(line)

    print(f"Finished rebuilding CKL file with new {key} added.")

def write_vkey_data(file_name, key, status, finding_details, comments):

    '''Write to CKL vKey'''

    vkeylist = []

    with open(file_name, 'r', encoding='utf-8') as ckl:
        ckl_dict = xmltodict.parse(ckl.read())

        for vuln in ckl_dict['CHECKLIST']['STIGS']['iSTIG']['VULN']:
            for attribute in vuln['STIG_DATA']:
                if str(attribute['ATTRIBUTE_DATA']).startswith('V-'):
                    vkeylist.append(attribute['ATTRIBUTE_DATA'])

    handler.check_file_and_keys(file_name=file_name, key=key, available_inputs=vkeylist)

    with open(file_name, 'r', encoding='utf-8') as ckl:
        ckl_dict = xmltodict.parse(ckl.read())

        for vuln in ckl_dict['CHECKLIST']['STIGS']['iSTIG']['VULN']:
            for attribute in vuln['STIG_DATA']:
                if attribute['ATTRIBUTE_DATA'] == key:
                    vuln['STATUS'] = status
                    vuln['FINDING_DETAILS'] = finding_details
                    vuln['COMMENTS'] = comments

    #Add to XML file
    with open(file_name, 'r+', encoding='utf-8') as ckl:

        xml_out = xmltodict.unparse(ckl_dict, pretty=True)
        ckl_file_data = [   '<?xml version="1.0" encoding="UTF-8"?>\n',
                            '<!--DISA STIG Viewer :: 2.17-->\n',
                            xml_out
                        ]
        ckl.writelines(ckl_file_data)

    #Create a list from all the lines to remove duplicate xml declarations
    with open(file_name, 'r', encoding='utf-8') as ckl:
        lines = ckl.readlines()

    #Rewrite the file without duplicate xml declaration
    with open(file_name, 'w', encoding='utf-8') as ckl:
        for number,line in enumerate(lines):
            if number not in [2]:
                ckl.writelines(line)



"""
Module providing error handling to prevent ckl file corruption and thaty it exists with 
the correct permissions
"""
import os

def check_file_and_keys(file_name, key, available_inputs):

    """Error out if the file is not found, is a directory, or incorrect permissions on the file"""
    try:
        with open(file_name, 'r', encoding='utf-8'):
            if os.path.getsize(file_name) == 0:
                raise ValueError('The CKL file is empty.')
        #file = open(file_name, encoding='utf-8')
    except FileNotFoundError:
        print(f'ERROR: File not found: {file_name}')
    except IsADirectoryError:
        print(f'ERROR: {file_name} is a directory and not a file.')
    except PermissionError:
        print(f'Permission denied for file: {file_name}')
    else:
        if key not in available_inputs:
            raise ValueError(f'{key} is not a valid key.')



"""
Module providing the ability to test the following:
    1. Read target data from ckl file.
    2. Write target data to the ckl file
    3. Write vkey data to the ckl file
    4. Load vkey data that can then be used to verify you are writing to an 
       actual vkey in that stig
    5. Load target data that can then be used to verify you are writing to 
       an actual target in that stig
"""

import unittest
import xmltodict
from stig_edit import ckl_editor

TEST_FILE = "test.ckl"

class TestStigEdit(unittest.TestCase):

    '''Class to go through and test each function within the ckl_editor module.'''

    def test_read_target_data(self):

        '''Test the read_target_data function'''

        result = ckl_editor.read_target_data(TEST_FILE)
        self.assertIsNotNone(result)

    def test_write_target_data(self):

        '''Test the write_target_data function'''

        key = 'ROLE'
        value = 'Member Server'
        #print(target_values)

        ckl_editor.write_target_data(   file_name=TEST_FILE,
                                        key=key,
                                        value=value
                                    )

        with open(TEST_FILE, 'r', encoding='utf-8') as ckl:
            ckl_dict = xmltodict.parse(ckl.read())
            self.assertEqual(ckl_dict['CHECKLIST']['ASSET'][key], value)

    def test_write_vkey_data(self):

        '''Test the write_vkey_data function'''

        key = 'V-230222'
        value = 'Not_Reviewed'
        finding_details = "Server was patched.\nThis is not a finding"
        comment = "This is a test comment."

        ckl_editor.write_vkey_data( file_name=TEST_FILE,
                                    key=key,
                                    status=value,
                                    finding_details=finding_details,
                                    comments=comment
                                )

        with open(TEST_FILE, 'r', encoding='utf-8') as ckl:
            ckl_dict = xmltodict.parse(ckl.read())

            for vuln in ckl_dict['CHECKLIST']['STIGS']['iSTIG']['VULN']:
                for attribute in vuln['STIG_DATA']:
                    if attribute['ATTRIBUTE_DATA'] == key:
                        self.assertEqual(vuln['STATUS'], value)
                        self.assertEqual(vuln['FINDING_DETAILS'], finding_details)
                        self.assertEqual(vuln['COMMENTS'], comment)
